import {
  createElement,
  Fragment,
  useMemo
} from 'react';
import { unified } from 'unified';
import rehypeParse from 'rehype-parse';
import rehypeReact from 'rehype-react';
import remarkParse from 'remark-parse';
import remarkGfm from 'remark-gfm';
import remarkRehype from 'remark-rehype';
import rehypeDocument from 'rehype-document';
import rehypeFormat from 'rehype-format';
import rehypeStringify from 'rehype-stringify';

import Node from '../Node';

const components = {
  h1: Node,
  h2: Node,
  h3: Node,
  h4: Node,
  h5: Node,
  h6: Node,
  p: Node,
  blockquote: Node,
  ol: Node,
  ul: Node,
  li: Node,
  a: Node,
  img: Node,
  b: Node,
  i: Node,
  code: Node,
};

const markdownParser = unified()
  .use(remarkParse)
  .use(remarkGfm)
  .use(remarkRehype)
  .use(rehypeDocument)
  .use(rehypeFormat)
  .use(rehypeStringify);

const htmlParser = unified()
  .use(rehypeParse, { fragment: true })
  .use(rehypeReact, {
    createElement,
    Fragment,
    components,
    passNode: true,
  });

const useRenderedText = function useRenderedText(text) {
  const Content = useMemo(() => {
    const html = markdownParser.processSync(text);
    const jsx = htmlParser.processSync(html.value);
    return jsx.result.props.children;
  }, [text]);

  return Content;
};

export default useRenderedText;
