import classnames from 'classnames';
import classes from './styles.module.scss';
import getSource from '../utils/getSource';
import useIsObserved from '../hooks/useIsObserved';
import useChangeSrcOnObserve from '../hooks/useChangeSrcOnObserve';
import useContainerPadding from '../hooks/useContainerPadding';

const defaultSizes = [420, 600, 768, 980, 1280, 1400, 2000];
const observationThreshold = 0;

const Image = function Image({
  src,
  mobileSrc,
  quality,
  alt,
  breakpoint,
  rootMargin,
  imageSizeOptions,
}) {
  const {
    height: intrinsicHeight,
    width: intrinsicWidth,
  } = imageSizeOptions;

  const sources = defaultSizes.map((size) => getSource({
    width: size,
    src,
    mobileSrc,
    quality,
    breakpoint,
  }));

  // lazyloading logic
  const [ref, observed] = useIsObserved({ observationThreshold, rootMargin });
  const [currentImgSrc, imgClass] = useChangeSrcOnObserve({
    observed,
    src,
    sizes: defaultSizes,
    classes,
  });

  // set aspect ratio padding
  const containerPadding = useContainerPadding({
    imageSizeOptions,
    breakpoint,
  });

  const containerStyle = {
    paddingTop: `${containerPadding}%`,
  };

  return (
    <picture
      className={classes.container}
      style={containerStyle}
    >

      {/* source for every size */}
      {observed && sources.map((props) => <source {...props} />)}

      {/* default low rez for blur up */}
      <img
        width={intrinsicWidth}
        height={intrinsicHeight}
        ref={ref}
        className={classnames(classes.root, imgClass)}
        alt={alt}
        src={currentImgSrc}
      />

    </picture>
  );
};
export default Image;
